==> GFS-4.1.0.tar.gz: GFS-4.1.0/src/tools/mkfs/main.c <==
/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**    Portions Copyright 1999-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: main.c
**
**    Description:
**	Called by mkfs(1M) or alone
**
*******************************************************************************
******************************************************************************/

#define _LARGEFILE64_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <assert.h>
#include <time.h>

#include "global.h"
#include "release.cf"
#include "pool.h"
#include "gfs_ondisk.h"

#define EXTERN
#include "mkfs_gfs.h"
#include "pool_geometry.h"
#include "fs_geometry.h"
#include "structures.h"



/*  Constants  */

#define OPTION_STRING               ("p:t:b:s:ij:J:Vhqd")





/******************************************************************************
*******************************************************************************
**
** print_usage()
**
** Description:
**   Print usage
**
*******************************************************************************
******************************************************************************/

static void print_usage()
{
  printf("\n");
  printf("Usage:\n");
  printf("\n");
  printf("%s [options] <device>\n", prog_name);
  printf("\n");
  printf("Basic options:\n");
  printf("\n");
  printf("  -p <name>       Name of the locking protocol\n");
  printf("  -t <name>       Name of the lock table\n");
  printf("  -b <bytes>      Filesystem block size\n");
  printf("  -s <blocks>     Journal segment size\n");
  printf("\n");
  printf("Non-Pool options:\n");
  printf("\n");
  printf("  -i              Ignore Pool data and use Non-Pool options\n");
  printf("  -j <num>        Number of journals\n");
  printf("  -J <MB>         Size of journals\n");
  printf("\n");
  printf("Fluff Options:\n");
  printf("\n");
  printf("  -V              Print program version information, then exit\n");
  printf("  -h              Print this help, then exit\n");
  printf("  -q              Don't print anything\n");
  printf("  -d              Enable debugging code\n");
}


/******************************************************************************
*******************************************************************************
**
** decode_arguments()
**
** Description:
**   Decode the command line arguments
**
** Input(s):
**   argc - well, it's argc...
**   argv - well, it's argv...
**
** Output(s):
**   comline - the returned arguments
**
*******************************************************************************
******************************************************************************/

static void decode_arguments(int argc, char *argv[], commandline_t *comline)
{
  int cont = TRUE;
  int optchar;


  while (cont)
  {
    optchar = getopt(argc, argv, OPTION_STRING);

    switch (optchar)
    {
    case 'p':
      if (strlen(optarg) >= GFS_LOCKNAME_LEN)
	die("lock protocol name %s is too long\n", optarg);
      strcpy(comline->lockproto, optarg);
      break;


    case 't':
      if (strlen(optarg) >= GFS_LOCKNAME_LEN)
	die("lock table name %s is too long\n", optarg);
      strcpy(comline->locktable, optarg);
      break;


    case 'b':
      comline->bsize = atoi(optarg);
      break;


    case 's':
      comline->seg_size = atoi(optarg);
      break;


    case 'i':
      comline->ignore_pool = TRUE;
      break;


    case 'j':
      comline->journals = atol(optarg);
      break;


    case 'J':
      comline->jsize = atol(optarg);
      break;


    case 'V':
      printf("mkfs_gfs %s (built %s %s)\n", GFS_RELEASE_NAME, __DATE__, __TIME__);
      printf("\n");
      printf("  Copyright 1995-1999 Regents of the University of Minnesota\n");
      printf("  Portions Copyright 2000-2001 Sistina Software, Inc.\n");
      exit(EXIT_SUCCESS);
      break;


    case 'h':
      print_usage();
      exit(EXIT_SUCCESS);
      break;


    case 'q':
      comline->quiet = TRUE;
      break;


    case 'd':
      comline->debug = TRUE;
      break;


    case ':':
    case '?':
      print_usage();
      exit(EXIT_FAILURE);      
      break;


    case EOF:
      cont = FALSE;
      break;


    default:
      die("Homer sleep now.\n");
      break;
    };
  }


  if (optind < argc)
  {
    comline->device = argv[optind];
    optind++;
  }
  else
    die("no device specified (try -h for help)\n");



  if (optind < argc) 
    die("Unrecognized option: %s\n", argv[optind]);


  if (comline->debug)
  {
    printf("Command Line Arguments:\n");
    printf("  proto = %s\n", comline->lockproto);
    printf("  table = %s\n", comline->locktable);
    printf("  bsize = %u\n", comline->bsize);
    printf("  seg_size = %u\n", comline->seg_size);
    printf("  ignore_pool = %u\n", comline->ignore_pool);
    printf("  journals = %u\n", comline->journals);
    printf("  jsize = %u\n", comline->jsize);
    printf("  debug = %d\n", comline->debug);
    printf("  device = %s\n", comline->device);
  }
}


/******************************************************************************
*******************************************************************************
**
** print_results()
**
** Description:
**   print results to the screen
**   Maybe this should look prettier
**
** Input(s):
**   comline - the command line arguments
**
*******************************************************************************
******************************************************************************/
	
void print_results(commandline_t *comline)
{
  if (comline->quiet)
    return;

  printf("Device:                    %s\n", comline->device);

  printf("Blocksize:                 %u\n", comline->bsize);
  printf("Filesystem Size:           %"FMT_64"u\n", comline->fssize);

  printf("Journals:                  %u\n", comline->journals);
  printf("Resource Groups:           %"FMT_64"u\n", comline->rgrps);

  printf("Locking Protocol:          %s\n", comline->lockproto);
  printf("Lock Table:                %s\n", comline->locktable);

  printf("\nSyncing...\n");
  sync();
  printf("All Done\n");
}


/******************************************************************************
*******************************************************************************
**
** main()
**
** Description:
**   Do everything
**
*******************************************************************************
******************************************************************************/

int main(int argc, char *argv[])
{
  commandline_t comline;
  pool_splist_t splist;
  rgrp_list_t *rlist;
  journal_list_t *jlist;

  unsigned int x;
  int error = 0;


  prog_name = argv[0];


  /*  Process the command line arguments  */

  memset(&comline, 0, sizeof(commandline_t));
  comline.bsize = MKFS_DEFAULT_BSIZE;
  comline.seg_size = MKFS_DEFAULT_SEG_SIZE;
  comline.jsize = MKFS_DEFAULT_JSIZE;

  decode_arguments(argc, argv, &comline);



  /*  Block sizes must be a power of two from 512 to 65536  */

  for (x = 512; x; x <<= 1)
    if (x == comline.bsize)
      break;

  if (!x || comline.bsize > 65536)
    die("block size must be a power of two between 512 and 65536\n");

  comline.sb_addr = GFS_SB_ADDR * GFS_BASIC_BLOCK / comline.bsize;

  if (comline.seg_size < 2)
    die("bad segment size\n");



  /*  Get the pool geometry  */

  memset(&splist, 0, sizeof(pool_splist_t));

  if (comline.ignore_pool)
    fake_pool_geometry(&comline, &splist);
  else
  {
    error = get_pool_geometry(&comline, &splist);
    if (error)
      fake_pool_geometry(&comline, &splist);
  }

  fix_pool_geometry(&comline, &splist);



  /*  Compute the resource group layouts  */

  rlist = compute_rgrp_layout(&comline, &splist);

  jlist = compute_journal_layout(&comline, &splist);


  /*  Start writing stuff out  */

  comline.fd = open(comline.device, O_RDWR | O_LARGEFILE);
  if (comline.fd < 0)
    die("can't open device %s\n", comline.device);


  write_sb(&comline, &splist, rlist);

  /*  Figure out where we start allocting in rgrp 0  */
  comline.rgrp0_next = comline.sb->sb_root_di.no_addr + 1;

  write_jindex(&comline, jlist);

  write_rindex(&comline, rlist);

  write_root(&comline);

  write_rgrps(&comline, rlist);

  write_journals(&comline, jlist);


  close(comline.fd);


  print_results(&comline);


  exit(EXIT_SUCCESS);
}

==> GFS-4.1.1.tar.gz: GFS-4.1.1/src/tools/mkfs/main.c <==
/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**    Portions Copyright 1999-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: main.c
**
**    Description:
**	Called by mkfs(1M) or alone
**
*******************************************************************************
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <assert.h>
#include <time.h>

#include "global.h"
#include "release.cf"
#include "pool.h"
#include "gfs_ondisk.h"

#define EXTERN
#include "mkfs_gfs.h"
#include "pool_geometry.h"
#include "fs_geometry.h"
#include "structures.h"



/*  Constants  */

#define OPTION_STRING               ("p:t:b:s:ij:J:Vhqd")





/******************************************************************************
*******************************************************************************
**
** print_usage()
**
** Description:
**   Print usage
**
*******************************************************************************
******************************************************************************/

static void print_usage()
{
  printf("\n");
  printf("Usage:\n");
  printf("\n");
  printf("%s [options] <device>\n", prog_name);
  printf("\n");
  printf("Basic options:\n");
  printf("\n");
  printf("  -p <name>       Name of the locking protocol\n");
  printf("  -t <name>       Name of the lock table\n");
  printf("  -b <bytes>      Filesystem block size\n");
  printf("  -s <blocks>     Journal segment size\n");
  printf("\n");
  printf("Non-Pool options:\n");
  printf("\n");
  printf("  -i              Ignore Pool data and use Non-Pool options\n");
  printf("  -j <num>        Number of journals\n");
  printf("  -J <MB>         Size of journals\n");
  printf("\n");
  printf("Fluff Options:\n");
  printf("\n");
  printf("  -V              Print program version information, then exit\n");
  printf("  -h              Print this help, then exit\n");
  printf("  -q              Don't print anything\n");
  printf("  -d              Enable debugging code\n");
}


/******************************************************************************
*******************************************************************************
**
** decode_arguments()
**
** Description:
**   Decode the command line arguments
**
** Input(s):
**   argc - well, it's argc...
**   argv - well, it's argv...
**
** Output(s):
**   comline - the returned arguments
**
*******************************************************************************
******************************************************************************/

static void decode_arguments(int argc, char *argv[], commandline_t *comline)
{
  int cont = TRUE;
  int optchar;


  while (cont)
  {
    optchar = getopt(argc, argv, OPTION_STRING);

    switch (optchar)
    {
    case 'p':
      if (strlen(optarg) >= GFS_LOCKNAME_LEN)
	die("lock protocol name %s is too long\n", optarg);
      strcpy(comline->lockproto, optarg);
      break;


    case 't':
      if (strlen(optarg) >= GFS_LOCKNAME_LEN)
	die("lock table name %s is too long\n", optarg);
      strcpy(comline->locktable, optarg);
      break;


    case 'b':
      comline->bsize = atoi(optarg);
      break;


    case 's':
      comline->seg_size = atoi(optarg);
      break;


    case 'i':
      comline->ignore_pool = TRUE;
      break;


    case 'j':
      comline->journals = atol(optarg);
      break;


    case 'J':
      comline->jsize = atol(optarg);
      break;


    case 'V':
      printf("mkfs_gfs %s (built %s %s)\n", GFS_RELEASE_NAME, __DATE__, __TIME__);
      printf("\n");
      printf("  Copyright 1995-1999 Regents of the University of Minnesota\n");
      printf("  Portions Copyright 2000-2001 Sistina Software, Inc.\n");
      exit(EXIT_SUCCESS);
      break;


    case 'h':
      print_usage();
      exit(EXIT_SUCCESS);
      break;


    case 'q':
      comline->quiet = TRUE;
      break;


    case 'd':
      comline->debug = TRUE;
      break;


    case ':':
    case '?':
      print_usage();
      exit(EXIT_FAILURE);      
      break;


    case EOF:
      cont = FALSE;
      break;


    default:
      die("Homer sleep now.\n");
      break;
    };
  }


  if (optind < argc)
  {
    comline->device = argv[optind];
    optind++;
  }
  else
    die("no device specified (try -h for help)\n");



  if (optind < argc) 
    die("Unrecognized option: %s\n", argv[optind]);


  if (comline->debug)
  {
    printf("Command Line Arguments:\n");
    printf("  proto = %s\n", comline->lockproto);
    printf("  table = %s\n", comline->locktable);
    printf("  bsize = %u\n", comline->bsize);
    printf("  seg_size = %u\n", comline->seg_size);
    printf("  ignore_pool = %u\n", comline->ignore_pool);
    printf("  journals = %u\n", comline->journals);
    printf("  jsize = %u\n", comline->jsize);
    printf("  debug = %d\n", comline->debug);
    printf("  device = %s\n", comline->device);
  }
}


/******************************************************************************
*******************************************************************************
**
** print_results()
**
** Description:
**   print results to the screen
**   Maybe this should look prettier
**
** Input(s):
**   comline - the command line arguments
**
*******************************************************************************
******************************************************************************/
	
void print_results(commandline_t *comline)
{
  if (comline->quiet)
    return;

  printf("Device:                    %s\n", comline->device);

  printf("Blocksize:                 %u\n", comline->bsize);
  printf("Filesystem Size:           %"FMT_64"u\n", comline->fssize);

  printf("Journals:                  %u\n", comline->journals);
  printf("Resource Groups:           %"FMT_64"u\n", comline->rgrps);

  printf("Locking Protocol:          %s\n", comline->lockproto);
  printf("Lock Table:                %s\n", comline->locktable);

  printf("\nSyncing...\n");
  sync();
  printf("All Done\n");
}


/******************************************************************************
*******************************************************************************
**
** main()
**
** Description:
**   Do everything
**
*******************************************************************************
******************************************************************************/

int main(int argc, char *argv[])
{
  commandline_t comline;
  pool_splist_t splist;
  rgrp_list_t *rlist;
  journal_list_t *jlist;

  unsigned int x;
  int error = 0;


  prog_name = argv[0];


  /*  Process the command line arguments  */

  memset(&comline, 0, sizeof(commandline_t));
  comline.bsize = MKFS_DEFAULT_BSIZE;
  comline.seg_size = MKFS_DEFAULT_SEG_SIZE;
  comline.jsize = MKFS_DEFAULT_JSIZE;

  decode_arguments(argc, argv, &comline);



  /*  Block sizes must be a power of two from 512 to 65536  */

  for (x = 512; x; x <<= 1)
    if (x == comline.bsize)
      break;

  if (!x || comline.bsize > 65536)
    die("block size must be a power of two between 512 and 65536\n");

  comline.sb_addr = GFS_SB_ADDR * GFS_BASIC_BLOCK / comline.bsize;

  if (comline.seg_size < 2)
    die("bad segment size\n");



  /*  Get the pool geometry  */

  memset(&splist, 0, sizeof(pool_splist_t));

  if (comline.ignore_pool)
    fake_pool_geometry(&comline, &splist);
  else
  {
    error = get_pool_geometry(&comline, &splist);
    if (error)
      fake_pool_geometry(&comline, &splist);
  }

  fix_pool_geometry(&comline, &splist);



  /*  Compute the resource group layouts  */

  rlist = compute_rgrp_layout(&comline, &splist);

  jlist = compute_journal_layout(&comline, &splist);


  /*  Start writing stuff out  */

  comline.fd = open(comline.device, O_RDWR);
  if (comline.fd < 0)
    die("can't open device %s\n", comline.device);


  write_sb(&comline, &splist, rlist);

  /*  Figure out where we start allocting in rgrp 0  */
  comline.rgrp0_next = comline.sb->sb_root_di.no_addr + 1;

  write_jindex(&comline, jlist);

  write_rindex(&comline, rlist);

  write_root(&comline);

  write_rgrps(&comline, rlist);

  write_journals(&comline, jlist);


  close(comline.fd);


  print_results(&comline);


  exit(EXIT_SUCCESS);
}
